#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

"""This module contains AWS CloudFormation Hook"""
from typing import Optional, Union

from boto3 import client, resource
from botocore.exceptions import ClientError

from airflow.providers.amazon.aws.hooks.base_aws import AwsBaseHook


class AWSCloudFormationHook(AwsBaseHook):
    """
    Interact with AWS CloudFormation.

    Additional arguments (such as ``aws_conn_id``) may be specified and
    are passed down to the underlying AwsBaseHook.

    .. seealso::
        :class:`~airflow.providers.amazon.aws.hooks.base_aws.AwsBaseHook`
    """

    def __init__(self, *args, **kwargs):
        super().__init__(client_type='cloudformation', *args, **kwargs)

    def get_stack_status(self, stack_name: Union[client, resource]) -> Optional[dict]:
        """Get stack status from CloudFormation."""
        self.log.info('Poking for stack %s', stack_name)

        try:
            stacks = self.get_conn().describe_stacks(StackName=stack_name)['Stacks']
            return stacks[0]['StackStatus']
        except ClientError as e:
            if 'does not exist' in str(e):
                return None
            else:
                raise e

    def create_stack(self, stack_name: str, params: dict) -> None:
        """
        Create stack in CloudFormation.

        :param stack_name: stack_name.
        :type stack_name: str
        :param params: parameters to be passed to CloudFormation.
        :type params: dict
        """
        if 'StackName' not in params:
            params['StackName'] = stack_name
        self.get_conn().create_stack(**params)

    def delete_stack(self, stack_name: str, params: Optional[dict] = None) -> None:
        """
        Delete stack in CloudFormation.

        :param stack_name: stack_name.
        :type stack_name: str
        :param params: parameters to be passed to CloudFormation (optional).
        :type params: dict
        """
        params = params or {}
        if 'StackName' not in params:
            params['StackName'] = stack_name
        self.get_conn().delete_stack(**params)



#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from typing import Any, Optional

try:
    from functools import cached_property
except ImportError:
    from cached_property import cached_property

from airflow.exceptions import AirflowException
from airflow.providers.amazon.aws.hooks.athena import AWSAthenaHook
from airflow.sensors.base import BaseSensorOperator


class AthenaSensor(BaseSensorOperator):
    """
    Asks for the state of the Query until it reaches a failure state or success state.
    If the query fails, the task will fail.

    :param query_execution_id: query_execution_id to check the state of
    :type query_execution_id: str
    :param max_retries: Number of times to poll for query state before
        returning the current state, defaults to None
    :type max_retries: int
    :param aws_conn_id: aws connection to use, defaults to 'aws_default'
    :type aws_conn_id: str
    :param sleep_time: Time in seconds to wait between two consecutive call to
        check query status on athena, defaults to 10
    :type sleep_time: int
    """

    INTERMEDIATE_STATES = (
        'QUEUED',
        'RUNNING',
    )
    FAILURE_STATES = (
        'FAILED',
        'CANCELLED',
    )
    SUCCESS_STATES = ('SUCCEEDED',)

    template_fields = ['query_execution_id']
    template_ext = ()
    ui_color = '#66c3ff'

    def __init__(
        self,
        *,
        query_execution_id: str,
        max_retries: Optional[int] = None,
        aws_conn_id: str = 'aws_default',
        sleep_time: int = 10,
        **kwargs: Any,
    ) -> None:
        super().__init__(**kwargs)
        self.aws_conn_id = aws_conn_id
        self.query_execution_id = query_execution_id
        self.sleep_time = sleep_time
        self.max_retries = max_retries

    def poke(self, context: dict) -> bool:
        state = self.hook.poll_query_status(self.query_execution_id, self.max_retries)

        if state in self.FAILURE_STATES:
            raise AirflowException('Athena sensor failed')

        if state in self.INTERMEDIATE_STATES:
            return False
        return True

    @cached_property
    def hook(self) -> AWSAthenaHook:
        """Create and return an AWSAthenaHook"""
        return AWSAthenaHook(self.aws_conn_id, sleep_time=self.sleep_time)



# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

"""Publish message to SQS queue"""
from typing import Optional

from airflow.models import BaseOperator
from airflow.providers.amazon.aws.hooks.sqs import SQSHook


class SQSPublishOperator(BaseOperator):
    """
    Publish message to a SQS queue.

    :param sqs_queue: The SQS queue url (templated)
    :type sqs_queue: str
    :param message_content: The message content (templated)
    :type message_content: str
    :param message_attributes: additional attributes for the message (default: None)
        For details of the attributes parameter see :py:meth:`botocore.client.SQS.send_message`
    :type message_attributes: dict
    :param delay_seconds: message delay (templated) (default: 1 second)
    :type delay_seconds: int
    :param aws_conn_id: AWS connection id (default: aws_default)
    :type aws_conn_id: str
    """

    template_fields = ('sqs_queue', 'message_content', 'delay_seconds', 'message_attributes')
    template_fields_renderers = {'message_attributes': 'json'}
    ui_color = '#6ad3fa'

    def __init__(
        self,
        *,
        sqs_queue: str,
        message_content: str,
        message_attributes: Optional[dict] = None,
        delay_seconds: int = 0,
        aws_conn_id: str = 'aws_default',
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.sqs_queue = sqs_queue
        self.aws_conn_id = aws_conn_id
        self.message_content = message_content
        self.delay_seconds = delay_seconds
        self.message_attributes = message_attributes or {}

    def execute(self, context):
        """
        Publish the message to SQS queue

        :param context: the context object
        :type context: dict
        :return: dict with information about the message sent
            For details of the returned dict see :py:meth:`botocore.client.SQS.send_message`
        :rtype: dict
        """
        hook = SQSHook(aws_conn_id=self.aws_conn_id)

        result = hook.send_message(
            queue_url=self.sqs_queue,
            message_body=self.message_content,
            delay_seconds=self.delay_seconds,
            message_attributes=self.message_attributes,
        )

        self.log.info('result is send_message is %s', result)

        return result
